@@ -19,6 +19,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* This file implement the functions needed to get dumb UNIX printing
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -38,6 +41,21 @@
 #include "i18n.h"
 #include "gtkutils.h"
 
+
+/*
+ * not all systems have defined __compar_fn_t, original definition
+ *  from glibc stdlib.h follows
+ */
+/* Shorthand for type of comparison functions.  */
+#ifndef __COMPAR_FN_T
+# define __COMPAR_FN_T
+typedef int (*__compar_fn_t) __PMT ((__const __ptr_t, __const __ptr_t));
+
+# ifdef	__USE_GNU
+typedef __compar_fn_t comparison_fn_t;
+# endif
+#endif
+
 typedef struct _PList PList;
 typedef struct _PDlgWidgets PDlgWidgets;
 typedef struct _PrintData PrintData;
@@ -83,8 +101,6 @@ struct _OutputData
   GtkWidget *print_dlg, *err_dlg, *parent_window;
 };
 
-/* Dumb UNIX printing */
-
 #define MAX_PLIST 8
 
 static int
@@ -97,13 +113,20 @@ void
 make_default_pr (PList *plist, int nbr_pr, char *defname)
 {
   int count;
+  gchar *printer_env, *new_default;
 
-  if (strlen (defname))
+  printer_env = getenv ("PRINTER");
+  if (printer_env)
+    new_default = printer_env;
+  else
+    new_default = defname;
+
+  if (strlen (new_default))
     {
       count = 0;
       while (count < nbr_pr)
 	{
-	  if (!strcmp (plist->name, defname))
+	  if (!strcmp (plist->name, new_default))
 	    plist->is_default = TRUE;
 	  plist++;
 	  count++;
@@ -119,7 +142,7 @@ get_printers (void)
   PList *plist;
 #if defined(LPC_COMMAND) || defined(LPSTAT_COMMAND)
   FILE *pfile;
-  char line[129], name[17];
+  char line[129];
 #endif	
 	
   defname[0] = '\0';
@@ -140,6 +163,8 @@ get_printers (void)
 	  {
 	    *strchr(line, ':') = '\0';
 	    strcpy(plist[plist_count].name, line);
+	    if (plist_count == 0)
+	      strcpy(defname, line);
 	    plist[plist_count].active = TRUE;
 	    plist[plist_count].is_default = FALSE;
 	    plist_count++;
@@ -161,7 +186,8 @@ get_printers (void)
 	      if (sscanf (line, "printer %s", name) == 1)
 		{
 		  strcpy (plist[plist_count].name, name);
-		  plist[plist_count].active = 1;
+		  plist[plist_count].active = TRUE;
+		  plist[plist_count].is_default = FALSE;
 		  plist_count++;
 		}
 	      else
